@@ -589,6 +589,10 @@ class Configuration(object):
             return None
 
         setup = _find_setup()
+        if setup is None:
+            msg("WARNING: Unable to find setup.h in {}, assuming {} is not available.".format(build_dir, flag))
+            return False
+
         with open(setup, 'rt') as f:
             for line in f:
                 if flag in line:
@@ -825,7 +829,7 @@ def getVcsRev():
     return svnrev
 
 
-def runcmd(cmd, getOutput=False, echoCmd=True, fatal=True):
+def runcmd(cmd, getOutput=False, echoCmd=True, fatal=True, onError=None):
     """
     Runs a give command-line command, optionally returning the output.
     """
@@ -867,6 +871,8 @@ def runcmd(cmd, getOutput=False, echoCmd=True, fatal=True):
         print("Command '%s' failed with exit code %d." % (cmd, rval))
         if getOutput:
             print(output)
+        if onError is not None:
+            onError()
         if fatal:
             sys.exit(rval)
 
@@ -959,12 +965,35 @@ def updateLicenseFiles(cfg):
     # Copy the license files from wxWidgets
     mkpath('license')
     for filename in ['preamble.txt', 'licence.txt', 'lgpl.txt', 'gpl.txt']:
-        copy_file(opj(cfg.WXDIR, 'docs', filename), opj('license',filename), update=1, verbose=1)
+        copy_file(opj(cfg.WXDIR, 'docs', filename), opj('license',filename),
+                      update=1, verbose=1)
+
+    # Get the sip license too
+    copy_file(opj('sip', 'siplib', 'LICENSE'), opj('license', 'sip-license.txt'),
+              update=1, verbose=1)
 
     # Combine the relevant files into a single LICENSE.txt file
     text = ''
-    for filename in ['preamble.txt', 'licence.txt', 'lgpl.txt']:
+    for filename in ['preamble.txt', 'licence.txt', 'lgpl.txt', 'sip-license.txt']:
         with open(opj('license', filename), 'r') as f:
             text += f.read() + '\n\n'
     with open('LICENSE.txt', 'w') as f:
         f.write(text)
+
+try:
+    from tempfile import TemporaryDirectory
+except ImportError:
+    from tempfile import mkdtemp
+
+    class TemporaryDirectory(object):
+        def __init__(self, suffix='', prefix='tmp', dir=None):
+            self.name = mkdtemp(suffix, prefix, dir)
+
+        def __enter__(self):
+            return self.name
+
+        def __exit__(self, exc, value, tb):
+            self.cleanup()
+
+        def cleanup(self):
+            shutil.rmtree(self.name)
