@@ -261,17 +261,27 @@ def fixWindowClass(klass, hideVirtuals=True, ignoreProtected=True):
         removeVirtuals(klass)
         addWindowVirtuals(klass)
 
-    if not klass.findItem('GetClassDefaultAttributes'):
-        klass.addItem(extractors.WigCode("""\
-            static wxVisualAttributes
-            GetClassDefaultAttributes(wxWindowVariant variant = wxWINDOW_VARIANT_NORMAL);
-            """))
-
     if not ignoreProtected:
         for item in klass.allItems():
             if isinstance(item, extractors.MethodDef) and item.protection == 'protected':
                 item.ignore(False)
 
+    fixDefaultAttributesMethods(klass)
+
+
+def fixDefaultAttributesMethods(klass):
+    if not klass.findItem('GetClassDefaultAttributes'):
+        m = extractors.MethodDef(
+            type='wxVisualAttributes', name='GetClassDefaultAttributes',
+            isStatic=True, protection='public',
+            items=[extractors.ParamDef(
+                type='wxWindowVariant', name='variant', default='wxWINDOW_VARIANT_NORMAL')]
+        )
+        klass.addItem(m)
+
+    if klass.findItem('GetDefaultAttributes'):
+        klass.find('GetDefaultAttributes').mustHaveApp()
+    klass.find('GetClassDefaultAttributes').mustHaveApp()
 
 
 def fixTopLevelWindowClass(klass, hideVirtuals=True, ignoreProtected=True):
@@ -308,6 +318,8 @@ def fixTopLevelWindowClass(klass, hideVirtuals=True, ignoreProtected=True):
             if isinstance(item, extractors.MethodDef) and item.protection == 'protected':
                 item.ignore(False)
 
+    fixDefaultAttributesMethods(klass)
+
 
 
 def fixSizerClass(klass):
@@ -1373,7 +1385,10 @@ def _generateDefineStub(code, define, typeValMap):
 
 def _generateGlobalStub(code, glob, typeValMap):
     code.hdr.append('extern {} {};'.format(glob.type, glob.name))
-    code.cpp.append('{} {};'.format(glob.type, glob.name))
+    if glob.type == 'const char*':
+        code.cpp.append('{} {} = "";'.format(glob.type, glob.name))
+    else:
+        code.cpp.append('{} {};'.format(glob.type, glob.name))
 
 
 def _generateEnumStub(code, enum, typeValMap):
